#include <stdio.h>
#include<vcg/complex/complex.h>

#include<vcg/simplex/face/topology.h>

// topology computation
#include<vcg/complex/algorithms/update/topology.h>

// normals
#include<vcg/complex/algorithms/update/normal.h>
#include<vcg/complex/algorithms/create/platonic.h>

#include<wrap/io_trimesh/export_off.h>

using namespace vcg;
using namespace std;

class MyEdge;
class MyFace;
class MyVertex;
struct MyUsedTypes : public UsedTypes<	Use<MyVertex>   ::AsVertexType,
                                        Use<MyEdge>     ::AsEdgeType,
                                        Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public Vertex<MyUsedTypes, vertex::Coord3f, vertex::Normal3f, vertex::BitFlags  >{};
class MyFace    : public Face< MyUsedTypes, face::FFAdj, face::VertexRef, face::Normal3f, face::BitFlags > {};
class MyEdge    : public Edge<MyUsedTypes>{};
class MyMesh    : public tri::TriMesh< vector<MyVertex>,
                                       vector<MyFace> , vector<MyEdge>  > {};

int main()
{
  MyMesh m;
  printf("Vertex %i triangle %i\n",m.VN(), m.FN());
  tri::Tetrahedron(m);
  printf("Vertex %i triangle %i\n",m.VN(), m.FN());
  printf("vert.size() %i\n", int(m.vert.size()));

  tri::UpdateNormals<MyMesh>::PerVertex(m);
  tri::UpdateNormals<MyMesh>::NormalizeVertex(m);
  tri::UpdateNormals<MyMesh>::PerFace(m);

  for(unsigned int i=0;i<m.vert.size();++i)
  {
    printf("Coord %f %f %f\n",m.vert[i].P()[0],m.vert[i].P().Y(),m.vert[i].P()[2]);
    printf("Norm  %f %f %f\n",m.vert[i].N()[0],m.vert[i].N().Y(),m.vert[i].N()[2]);
  }

  tri::Allocator<MyMesh>::DeleteFace(m,m.face[0]);
  tri::Allocator<MyMesh>::DeleteFace(m,m.face[1]);
  tri::Allocator<MyMesh>::CompactFaceVector(m);

  tri::UpdateTopology<MyMesh>::FaceFace(m);

  for(unsigned int i=0;i<m.face.size();++i)
  {
    printf("---\n");
    printf("%lu %lu %lu\n",
           tri::Index(m,m.face[i].V(0)),
           tri::Index(m,m.face[i].V(1)),
           tri::Index(m,m.face[i].V(2)) );
    printf("Norm  %f %f %f\n",m.face[i].N()[0],m.face[i].N().Y(),m.face[i].N()[2]);
    printf("adj  %lu %lu %lu\n",
           tri::Index(m,m.face[i].FFp(0)),
           tri::Index(m,m.face[i].FFp(1)),
           tri::Index(m,m.face[i].FFp(2)));
    printf("boundary %c %c %c\n",
           face::IsBorder(m.face[i],0)?'y':'n',
           face::IsBorder(m.face[i],1)?'y':'n',
           face::IsBorder(m.face[i],2)?'y':'n');
  }

  tri::io::ExporterOFF<MyMesh>::Save(m,"test.off");

  return 0;
}

